{ ****************************************************************************
    Android application for managing absentees. Visual part.
  ****************************************************************************}

unit absenteeapp;

{$mode objfpc}{$H+}
{$namespace eu.blaisepascal.absenteeapp}
{$modeswitch unicodestrings}

interface

uses
  androidr14, Resources, absenteedata, imageloader;

type
  TNextProcedure = Procedure of Object;

  TPupilItem = Class
    Text : AWTextView;
    Checkbox : AWCheckbox;
  end;

  { TIDTag }

  TIDTag = Class
    ID : JInt;
    constructor Create(AID : JInt);
  end;

  { TCheckBoxListBinder }

  TCheckBoxListBinder = Class(JLObject,AWSimpleCursorAdapter.InnerViewBinder)
    FListener : AWCompoundButton.InnerOnCheckedChangeListener;
    function setViewValue(view: AVView; cursor: ADCursor; colindex : jint) : jboolean;
  end;

  TAbsenteePreferenceActivity = class(APPreferenceActivity)
  public
    procedure onCreate(savedInstanceState: AOBundle); override;
  end;

  { TPupilsActivity }

  TPupilsActivity = class(AAListActivity,
                          AWCompoundButton.InnerOnCheckedChangeListener)
  private
    fID: jlong;
  protected
    fDataHelper: TAbsenteeDBHelper;
    fadapter : AWSimpleCursorAdapter;
    fSelectedID: jlong;
    fSortFirstName: boolean;
    procedure FillItems(Isupdate: Boolean);
  public
    procedure onCreate(savedInstanceState: AOBundle); override;
    procedure onCheckedChanged(cb: AWCompoundButton; checked: jboolean);
  end;

  { TGroupActivity }
  TGroupActivity = class(AAListActivity,
    AWAdapterView.InnerOnItemClickListener)
  private
    procedure CancelImport(S: String);
    procedure ImportGroups(DoNext: TNextProcedure);
    procedure ImportPupils;
    procedure SaveNewGroups(S: String);
    procedure SaveNewPupils(S: String);
    procedure SavePupils(S: String);
  protected
    FImportTask : TDownloadHelperTask;
    fAdapter: AWSimpleCursorAdapter;
    fDataHelper: TAbsenteeDBHelper;
    fSelectedID: jlong;
    FGroupsDescending : Boolean;
    FPupilsFirstName : Boolean;
    FImporting : AAProgressDialog;
    FDoNext:TNextProcedure;
    procedure FillItems(isRefresh : Boolean);
  public
    Procedure SaveSetting(Const AName : String; AValue : Boolean);
    procedure onCreate(savedInstanceState: AOBundle); override;
    Function onCreateOptionsMenu(AMenu : AVMenu) :  boolean ; override;
    Function onOptionsItemSelected(AItem : AVMenuItem) : Boolean; override;
    procedure onItemClick(aParent: AWAdapterView; aView: AVView; aPosition: jint; aID: jlong);
  end;

implementation

{ TCheckBoxListBinder }

function TCheckBoxListBinder.setViewValue(view: AVView; cursor: ADCursor;
  colindex: jint): jboolean;

Var
  cb : AWCheckbox;
  llid,i : jint;
  s : string;

begin
  Result:=(colIndex=2);
  if Result then
    begin
    cb:=AWCheckBox(view);
    llid:=cursor.getint(0);
    cb.setTag(TIDTag.Create(llid));
    i:=cursor.getColumnIndex(TAbsenteeDBHelper.Columnpresencepresence);
    if not Cursor.IsNull(i) then
      begin
      s:=cursor.getstring(i);
      cb.setChecked(s='+');
      end;
    cb.setOnCheckedChangeListener(flistener);
    end;
end;

{ TIDTag }

constructor TIDTag.Create(AID : JInt);
begin
  ID:=AID;
end;


procedure TPupilsActivity.onCheckedChanged(cb: AWCompoundButton;
  checked: jboolean);

Var
  LLID : Integer;

begin
  LLID:=TIDTag(cb.getTag()).ID;
  FDataHelper.Pupilpresence[LLID]:=cb.isChecked;
end;

{ TPupilsActivity }

procedure TPupilsActivity.FillItems(Isupdate: Boolean);

var
  cur: ADCursor;
  colfrom: array[0..1] of JLString;
  b : TCheckBoxListBinder;

begin
  if IsUpdate then
    fAdapter.getCursor.requery
  else
    begin
    cur:=fDataHelper.GetPupilPresenceFromGroup(FID,fSortFirstName);
    startManagingCursor(cur);
    colfrom[0] := TAbsenteeDBHelper.columnPupilName;
    colfrom[1] := TAbsenteeDBHelper.Columnpresencepresence;
    fAdapter := AWSimpleCursorAdapter.Create(Self,
                  R.layout.pupil_list_item, cur, colfrom,
                  [R.id.pupil_name,R.id.present]);
    b:=TCheckBoxListBinder.create;
    b.flistener:=Self;
    fadapter.setViewBinder(b);
    setListAdapter(fAdapter);
    end;
end;

procedure TPupilsActivity.onCreate(savedInstanceState: AOBundle);

begin
  inherited onCreate(savedInstanceState);
  fDataHelper := TAbsenteeDBHelper.Create(Self);
  fID := getIntent.getLongExtra('eu.blaisepascal.absenteeapp.ID', 0);
  fSortFirstName:=getIntent.getBooleanExtra('eu.blaisepascal.absenteeapp.SortFirstName',False);
  if fID = 0 then
    raise EAbsenteeData.Create('No ID for absentee app given');
  setTitle(R.strings.name_pupils);
  FillItems(False);
end;


{$R *.res}

{.$R *.res}


{ TGroupActivity }

procedure TGroupActivity.FillItems(isRefresh : Boolean);
var
  cur: ADCursor;
  colfrom: array[0..0] of JLString;
begin
  if Not isRefresh then
    begin
    cur := fDataHelper.GetAllGroups(FGroupsDescending);
    startManagingCursor(cur);
    colfrom[0]:=TAbsenteeDBHelper.ColumnGroupName;
    fAdapter:=AWSimpleCursorAdapter.Create(Self,
                R.layout.group_list_item, cur, colfrom,
               [R.id.group_name]);
    setListAdapter(fAdapter);
    end
  else
    fAdapter.getCursor.reQuery;
end;

Const
  PrefGroups = 'GroupsDescending';
  PrefPupils = 'PupilsFirstName';

procedure TGroupActivity.onCreate(savedInstanceState: AOBundle);

Var
  Prefs : ACSharedPreferences;

begin
  inherited onCreate(savedInstanceState);
  Prefs:=APPreferenceManager.getDefaultSharedPreferences(GetBaseContext());
  FGroupsDescending:=Prefs.getBoolean(PrefGroups,False);
  FPupilsFirstName:=Prefs.getBoolean(PrefPupils,False);
  fDataHelper := TAbsenteeDBHelper.Create(Self);
  setTitle(R.strings.name_groups);
  FillItems(False);
  getListView.setOnItemClickListener(Self);
end;

procedure TGroupActivity.onItemClick(aParent: AWAdapterView; aView: AVView;
  aPosition: jint; aID: jlong);
var
  intent: ACIntent;
begin
  if aParent.getContext <> Self then
    Exit;
  intent := ACIntent.Create(Self, JLClass(TPupilsActivity));
  intent.putExtra('eu.blaisepascal.absenteeapp.ID', aID);
  intent.putExtra('eu.blaisepascal.absenteeapp.SortFirstName',FPupilsFirstName);
  startActivity(intent);
end;

function TGroupActivity.onCreateOptionsMenu(AMenu: AVMenu): boolean;

begin
  inherited;
  AMenu.add(0,AVMenu.FIRST,0,R.strings.Preferences);
  AMenu.add(0,AVMenu.FIRST+1,0,R.strings.Synchronize);
  Result:=True;
end;

Procedure TGroupActivity.SaveSetting(Const AName : String; AValue : Boolean);

Var
  PrefsEditor : ACSharedPreferences.InnerEditor;

begin
  PrefsEditor:=APPreferenceManager.getDefaultSharedPreferences(GetBaseContext()).Edit();
  PrefsEditor.putBoolean(AName,AValue);
  PrefsEditor.commit();
end;

Procedure TGroupActivity.SavePupils(S : String);

begin
  FImporting.setMessage(JLString.Create('Saving pupils locally'));
  try
    fDataHelper.ImportPupils(S);
    FImporting.dismiss();
  except
    CancelImport('An error occured while saving the pupils.');
  end;
end;

Procedure TGroupActivity.ImportPupils;

Var
  URL : JLString;

begin
  if Not Assigned(FImporting) then
    FImporting:=AAProgressDialog.create(Self);
  FImporting.setMessage(JLString.Create('Fetching pupils'));
  FImporting.setIndeterminate(True);
  FImporting.setCancelable(False);
  FImporting.show();
  FImportTask:=TDownloadHelperTask.Create;
  FImportTask.OnError:=@CancelImport;
  FImportTask.OnSuccess:=@SavePupils;
  URL:=JLString.Create('http://192.168.0.98:60080/~michael/pupils.json');
  FImportTask.execute([URL]);
end;

Procedure TGroupActivity.CancelImport(S : String);

Var
  Bld : AAAlertDialog.InnerBuilder;
  Dlg : AAAlertDialog;
  JS : JLString;

begin
  FImportTask:=Nil;
  If FImporting<>Nil then
    FImporting.Dismiss();
  Bld:=AAAlertDialog.InnerBuilder.Create(self);
  JS:=JLString.Create('Synchronization failed: '+S);
  Bld.setMessage(JS);
  Bld.setCancelable(true);
  Dlg:=Bld.create_();
  Dlg.show();
end;

Procedure TGroupActivity.SaveNewGroups(S : String);

begin
  FImportTask:=Nil;
  if (S='') or (Pos('{',S)<>1) then
    CancelImport('No valid groups JSON from server')
  else
    begin
    FImporting.SetMessage(JLString.Create('Saving groups'));
    try
      fDataHelper.ImportGroups(S);
      FillItems(False);
    except
      CancelImport('An error occurred while saving the groups');
      exit;
    end;
    if Assigned(FDoNext) then
      FDoNext;
    end;
end;

Procedure TGroupActivity.SaveNewPupils(S : String);

begin
  FImportTask:=Nil;
  if (S='') or (Pos('{',S)<>1) then
    CancelImport('No valid pupils JSON from server');
  FImporting.SetMessage(JLString.Create('Saving pupils'));
  fDataHelper.ImportPupils(S);
  FImporting.Cancel;
end;

Procedure TGroupActivity.ImportGroups(DoNext : TNextProcedure);

Var
  URL : JLString;

begin
  FDoNext:=DoNext;
  FImporting:=AAProgressDialog.create(Self);
  FImporting.setMessage(JLString.Create('Fetching groups'));
  FImporting.setIndeterminate(True);
  FImporting.setCancelable(False);
  FImporting.show();
  FImportTask:=TDownloadHelperTask.Create();
  FImportTask.OnError:=@CancelImport;
  FImportTask.OnSuccess:=@SaveNewGroups;
  URL:=JLString.Create('http://192.168.0.98:60080/~michael/groups.json');
  FImportTask.execute([URL]);
end;

function TGroupActivity.onOptionsItemSelected(AItem: AVMenuItem): Boolean;

var
  intent: ACIntent;

begin
  AItem.setChecked(not AItem.IsChecked());
  if AItem.getItemId()=AVMenu.First then
    begin
    intent := ACIntent.Create(Self, JLClass(TAbsenteePreferenceActivity));
    startActivity(intent);
    Exit(True);
    end
  else if AItem.getItemId()=AVMenu.First+1 then
    begin
    ImportGroups(@ImportPupils);
    Exit(True);
    end
  else
    Result:=Inherited onOptionsItemSelected(AItem);
end;

Procedure TAbsenteePreferenceActivity.onCreate(savedInstanceState: AOBundle);

begin
  inherited onCreate(savedInstanceState);
  addPreferencesFromResource(R.layout.prefs);
end;

end.

