{ ****************************************************************************
    Android application for managing absentees. Database backend.
  ****************************************************************************}

unit absenteedata;

{$namespace eu.blaisepascal.absenteeapp}
{$mode objfpc}{$H+}
{$modeswitch unicodestrings}

interface

uses
  AndroidR14, jsonobject;

type
  EAbsenteeData = class(JLException)
  end;

  TOnSuccessHandler = Procedure(AResult : String) Of Object;

  { TDownloadHelperTask }

  TDownloadHelperTask = Class(AOAsyncTask)
  private
    FOnError: TOnSuccessHandler;
    FOnSuccess: TOnSuccessHandler;
    FErrorMsg : String;
    function downloadUrl(AURL: String): String;
    function ReadAsString(ins: JIInputStream): String;
  Protected
     Function doInBackground(const para1: array of JLObject): JLObject; override;
     Procedure onPostExecute(para1: JLObject); override;
  Public
     Property OnSuccess : TOnSuccessHandler Read FOnSuccess Write FOnSuccess;
     Property OnError : TOnSuccessHandler Read FOnError Write FOnError;
  end;

  { TAbsenteeDBHelper }

  TAbsenteeDBHelper = class(ADSSqliteOpenHelper)
  strict private
    procedure EnableWIFI(AContext: ACContext);
    function GetLocalGroupID(DB: ADSSqliteDatabase; ARemoteID: Integer): Integer;
    function GetLocalPupilID(DB: ADSSqliteDatabase; ARemoteID: Integer): Integer;
    function GetNameOfGroup(aID: jlong): String;
    procedure ImportGroup(DB: ADSSqliteDatabase; AID: Integer; AName: string);
    procedure ImportPupil(DB : ADSSqliteDatabase; AID: Integer; AFirstName, ALastName: string; AGroupID : Integer);
    procedure PopulateDatabase(aDatabase: ADSSqliteDatabase);
    procedure SetNameOfGroup(aID: jlong; aValue: String);
    function GetNameOfPupil(aID: jlong): String;
    function GetPupilpresence(aID: jlong): boolean;
    procedure SetPupilpresence(aID: jlong; aValue: boolean);
  private
    const
      DatabaseVersion = 1;
      DatabaseName = 'absentees';
      CreateTableGroup = 'create table Groups (' +
                               '  gr_id integer primary key autoincrement,' +
                               '  gr_remote_id integer,' +
                               '  gr_name varchar not null' +
                               ')';
      CreateTablePupil = 'create table Pupil (' +
                             '  pu_id integer primary key autoincrement,' +
                             '  pu_remote_id integer,' +
                             '  pu_firstname varchar not null, ' +
                             '  pu_lastname varchar not null' +
                             ')';
      CreateTablePupilGroup = 'create table PupilGroup (' +
                         '  pg_id integer primary key autoincrement,' +
                         '  pg_Pupil_fk integer not null,' +
                         '  pg_Group_fk integer not null,' +
                         '  foreign key(pg_Pupil_fk) references Pupil (pu_id)' +
                         '    on delete cascade,' +
                         '  foreign key(pg_Group_fk) references Groups (gr_id)' +
                         '    on delete cascade' +
                         ')';
      CreateTablepresence = 'create table presence (' +
                             '  pr_id integer primary key autoincrement,' +
                             '  pr_Pupil_fk id not null,' +
                             '  pr_date date not null default CURRENT_DATE,' +
                             '  pr_code integer, '+
                             '  foreign key(pr_Pupil_fk) references Pupil(pu_id)' +
                             '    on delete cascade' +
                             ')';
  public
    const
      ColumnGroupName = 'gr_name';
      ColumnPupilFirstName = 'pu_firstname';
      ColumnPupilLastName = 'pu_lastname';
      columnPupilName = 'pu_name';
      ColumnGroupID = 'gr_id';
      ColumnPupilID = 'pu_id';
      ColumnpresenceID = 'pr_id';
      ColumnpresencePupilfk = 'pr_Pupil_fk';
      Columnpresencepresence = 'pr_code';
      TableGroup = 'Groups';
      TablePupil = 'Pupil';
      Tablepresence = 'presence';

  public
    constructor Create(aContext: ACContext);
    procedure onCreate(aDatabase: ADSSqliteDatabase); override;
    procedure onUpgrade(aDatabase: ADSSqliteDatabase;
      aOldVersion, aNewVersion: jint); override;
    procedure onOpen(aDatabase: ADSSqliteDatabase); override;

    function GetAllGroups(sortdesc : Boolean): ADCursor;
    function GetPupilsFromGroup(aGroup: jlong; sortfirstname : Boolean): ADCursor;
    function GetPupilpresenceFromGroup(aGroup: jlong; sortfirstname : Boolean): ADCursor;

    procedure DeletePupil(aID: jlong);
    procedure DeleteGroup(aID: jlong);
    procedure Deletepresence(aID: jlong);
    // HTTP
    procedure FetchJSON(AURL: String; OnSucces,OnError: TOnSuccessHandler);
    Function HaveInternetConnection(AContext : ACContext): Boolean;
    Procedure ImportPupils(AJSON : String);
    Procedure ImportGroups(AJSON : String);
  public
    property GroupNaam[aID: jlong]: String read GetNameOfGroup write SetNameOfGroup;
    property PupilNaam[aID: jlong]: String read GetNameOfPupil ;
    property Pupilpresence[aID: jlong]: Boolean read GetPupilpresence  write SetPupilpresence;
  end;

implementation

function TDownloadHelperTask.doInBackground(const para1: array of JLObject): JLObject;

Var
  S,Res : String;

begin
  Result:=Nil;
  FErrorMsg:='';
  Res:='';
  S:='';
  try
    S:=JLString(para1[0]);
    Res:=DownloadUrl(S);
    Result:=JLString.Create(Res);
  except
    On e : JLThrowable do
      FErrorMsg:='Unable to retrieve web page: URL may be invalid.';
  end;
end;

Function TDownloadHelperTask.downloadUrl(AURL: String): String;

Var
   Ins : JIInputStream;
   Conn : JNHttpURLConnection;
   url : JNURL;

begin
  ins:=Nil;
  URL:=Nil;
  Conn:=Nil;
  try
    url:=JNURL.Create(JLString.Create(AURL));
    conn:=url.openConnection() as JNHttpURLConnection;
    conn.setReadTimeout(10000);
    conn.setConnectTimeout(15000);
    conn.setRequestMethod('GET');
    conn.setDoInput(true);
    conn.connect();
    if (conn.getResponseCode()=200) then
      begin
      ins:=conn.getInputStream();
      Result:=ReadAsString(ins);
      end
    else
      Result:='';
  finally
    if ins<>nil then
      ins.close();
  end;
end;

function TDownloadHelperTask.ReadAsString(ins: JIInputStream): String;

Var
  Reader : JIBufferedReader;
  SB : JLStringBuilder;
  L : JLString;

begin
  Reader:=JIBufferedReader.Create(JIInputStreamReader.Create(ins,'UTF-8'));
  SB:=JLStringBuilder.Create;
  Result:='';
  L:=Reader.readLine;
  while L<>Nil do
    begin
    sb.append(L);
    sb.append(#10);
    L:=Reader.readLine;
    end;
  Result:=sb.ToString;
end;

procedure TDownloadHelperTask.onPostExecute(para1: JLObject);

Var
  S : String;

begin
  S:=JLString(Para1);
  if (FErrorMsg<>'') then
    begin
    If Assigned(OnError) then
      OnError(FErrorMsg);
    end
  else If Assigned(OnSuccess) then
    OnSuccess(S);
end;


{ TAbsenteeDBHelper }

constructor TAbsenteeDBHelper.Create(aContext: ACContext);
begin
  inherited Create(aContext, DatabaseName, Nil, DatabaseVersion);
end;

procedure TAbsenteeDBHelper.PopulateDatabase(aDatabase: ADSSqliteDatabase);

Const
  LLFirst : Array[1..12] of String =
          ('Andre','Bieke','Celine','Dirk','Elodie','Frouke',
           'Guy','Herman','Ines','Jo','Kurt','Louise');
  LLLast : Array[1..12] of String =
          ('Anthonissen','Bervoets','Courtois','Dedecker','Enfoire','Foudetoi',
           'Guylain','Horsdoeuvre','Ildefonse','Janplezier','Karmelitus','Levoisin');

Var

  I : integer;
  m,n,q : string;

begin

    // Klassen
    For I:=1 to 6 do
      begin
      n := JLLong.Create(i).toString;
      Q:='insert into Groups (gr_name) values (''Group '+n+'A'')';
      aDatabase.ExecSQL(Q);
      Q:='insert into Groups (gr_name) values (''Group '+n+'B'')';
      aDatabase.ExecSQL(Q);
      end;
    // Pupils
    For I:=1 to 12 do
      begin
      Q:='insert into Pupil (pu_firstname,pu_lastname) values ('''+llFirst[i]+''', '''+LLLast[i]+''')';
      aDatabase.ExecSQL(Q);
      Q:='insert into Pupil (pu_firstname,pu_lastname) values ('''+llFirst[13-i]+''', '''+LLLast[i]+''')';
      aDatabase.ExecSQL(Q);
      end;
    // Put pupils in groups
    For I:=1 to 24 do
      begin
      n := JLLong.Create(i).toString;
      m := JLLong.Create(i div 2).toString;
      Q:='insert into PupilGroup (pg_Pupil_fk, pg_Group_fk) values ('+n+' , '+m+')';
      aDatabase.ExecSQL(Q);
      end;
end;

procedure TAbsenteeDBHelper.onCreate(aDatabase: ADSSqliteDatabase);


begin
  aDatabase.execSQL(CreateTableGroup);
  aDatabase.execSQL(CreateTablePupil);
  aDatabase.execSQL(CreateTablePupilGroup);
  aDatabase.execSQL(CreateTablepresence);
end;

procedure TAbsenteeDBHelper.onUpgrade(aDatabase: ADSSqliteDatabase;
  aOldVersion, aNewVersion: jint);
begin
  // nothing
end;

procedure TAbsenteeDBHelper.onOpen(aDatabase: ADSSqliteDatabase);
begin
  inherited onOpen(aDatabase);
  if not aDatabase.isReadOnly then
    aDatabase.ExecSQL('pragma foreign_keys=on;');
end;

function TAbsenteeDBHelper.GetPupilsFromGroup(aGroup: jlong; sortfirstname : Boolean): ADCursor;

var
  query: String;

begin
  query := JLLong.Create(aGroup).toString;
  query := 'select pu_id as _id, pu_firstname || '' '' || pu_lastname as pu_name'+
           ' from '+
           ' PupilGroup left join Pupil on (pg_Pupil_fk=pu_id)'+
           ' where (pg_Group_fk='+Query+') order by ';
  if sortfirstname then
    query := query + 'pu_firstname asc'
  else
    query := query + 'pu_lastname asc';
  Result := getWritableDatabase.rawQuery(query,nil);
end;

function TAbsenteeDBHelper.GetPupilpresenceFromGroup(aGroup: jlong; sortfirstname : Boolean): ADCursor;

var
  query: String;

begin
  query := JLLong.Create(aGroup).toString;
  query := 'select pu_id as _id, pu_firstname || '' '' || pu_lastname as pu_name, pr_code' +
           ' from ' +
           ' PupilGroup left join Pupil on (pg_Pupil_fk=pu_id)' +
           ' left join presence on ((pr_Pupil_fk=pu_id) and (pr_date=date(''now''))) '+
           ' where (pg_Group_fk='+Query+') order by ';
  if sortfirstname then
    query := query + 'pu_firstname asc'
  else
    query := query + 'pu_lastname asc';
  Result := getWritableDatabase.rawQuery(query,nil);
end;

function TAbsenteeDBHelper.GetAllGroups(sortdesc : Boolean): ADCursor;
var
  query: String;
begin
  query := 'select gr_id as _id, gr_name from Groups order by gr_name ';
  if SortDesc then
    query := query + ' desc'
  else
    query := query + ' asc';
  Result := getWritableDatabase.rawQuery(query, Nil);
end;



function TAbsenteeDBHelper.GetNameOfPupil(aID: jlong): String;
var
  cur: ADCursor;
  id: String;

begin
  id := JLLong.Create(aID).toString;
  cur := getWritableDatabase.rawQuery(
           'select pu_firstname || '' '' || pu_lastname from Pupil where pu_id = ' + id, Nil);
  if cur.getCount = 0 then
    raise EAbsenteeData.Create('Pupil not found');
  cur.moveToFirst;
  Result := cur.getString(0);
end;

function TAbsenteeDBHelper.GetPupilpresence(aID: jlong): boolean;
var
  cur: ADCursor;
  id, sql: String;
  query: JLString;

begin
  id := JLLong.Create(aID).toString;
  sql := 'select pr_code from presence where (pr_pupil_fk='+id+')'+
         'and (pr_date=date(''now''))';
  cur := getWritableDatabase.rawQuery(sql,nil);
  if cur.getCount = 0 then
    Result:=False
  else
    begin
    cur.moveToFirst;
    sql:='+';
    query:=JLString.Create('+');
    Result := (cur.getString(0)=query);
    end;
end;

procedure TAbsenteeDBHelper.SetPupilpresence(aID: jlong; aValue: boolean);
var
  cur: ADCursor;
  id, sql: String;
  db : ADSSQLiteDatabase;

begin
  id := JLLong.Create(aID).toString;
  db:=getWritableDatabase;
  sql := 'select pr_id from presence where (pr_pupil_fk='+id+')'+
         'and (pr_date=date(''now''))';
  AULog.d('absentee',sql);
  cur := db.rawQuery(sql,nil);
  if (cur.getCount=0) then
    begin
    sql:='insert into presence (pr_pupil_fk, pr_date, pr_code) '+
         ' values ('+id+',date(''now''),';
    if aValue then
      sql:=sql+'''+'')'
    else
      sql:=sql+'''-'')';
    AULog.d('absentee','inserting : '+sql);
    db.execSQL(sql);
    end
  else
    begin
    cur.moveToFirst;
    id := JLLong.create(cur.getint(0)).tostring;
    cur.close;
    sql:='update presence set pr_code=';
    if aValue then
      sql:=sql+'''+'''
    else
      sql:=sql+'''-''';
    sql:=sql+' where (pr_id = '+id+')';
    AULog.d('absentee','Updating : '+sql);
    db.execsql(sql);
    end;
end;

procedure TAbsenteeDBHelper.SetNameOfGroup(aID: jlong; aValue: String);
var
  where: String;
  values: ACContentValues;
begin
  where := JLLong.Create(aID).toString;
  where := 'gr_id = ' + where;
  values := ACContentValues.create;
  values.put(ColumnGroupName, aValue);
  getWritableDatabase.update(TableGroup, values, where, Nil);
end;

function TAbsenteeDBHelper.GetNameOfGroup(aID: jlong): String;
var
  cur: ADCursor;
  id: String;

begin
  id := JLLong.Create(aID).toString;
  cur := getWritableDatabase.rawQuery(
           'select gr_name from Groups where gr_id = ' + id, Nil);
  if cur.getCount = 0 then
    raise EAbsenteeData.Create('Group not found');
  cur.moveToFirst;
  Result := cur.getString(0);
end;

procedure TAbsenteeDBHelper.DeleteGroup(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := 'gr_id = ' + where;
  getWritableDatabase.delete(TableGroup, where, Nil);
end;

procedure TAbsenteeDBHelper.DeletePupil(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := 'pu_id = ' + where;
  getWritableDatabase.delete(TablePupil, where, Nil);
end;

procedure TAbsenteeDBHelper.Deletepresence(aID: jlong);
var
  where: String;
begin
  where := JLLong.Create(aID).toString;
  where := '_id = ' + where;
  getWritableDatabase.delete(Tablepresence, where, Nil);
end;

procedure TAbsenteeDBHelper.FetchJSON(AURL: String; OnSucces,OnError: TOnSuccessHandler );

Var
  T : TDownloadHelperTask;

begin
  T:=TDownloadHelperTask.Create;
  T.OnSuccess:=OnSucces;
  T.OnError:=OnError;
  T.execute([JLString.Create(AURL)]);
end;

function TAbsenteeDBHelper.HaveInternetConnection(AContext : ACContext): Boolean;

Var
  connMgr : ANConnectivityManager;
  Info : ANNetworkInfo;

begin
  connMgr:=ANConnectivityManager(AContext.getSystemService(ACContext.CONNECTIVITY_SERVICE));
  info:=connMgr.getActiveNetworkInfo();
  Result:=Assigned(Info) and (Info.isConnected()=True);
end;

Procedure TAbsenteeDBHelper.EnableWIFI(AContext : ACContext);

Var
  wifiMgr: ANWWifiManager;

begin
  wifiMgr:=ANWWifiManager(Acontext.getSystemService(ACContext.WIFI_SERVICE));
  wifiMgr.setWifiEnabled(True);
end;

procedure TAbsenteeDBHelper.ImportPupil(DB : ADSSqliteDatabase; AID : Integer; AFirstName,ALastName : string; AGroupID : Integer);

Var
  Q : String;
  SID,GID : String;
  AGID,ALID : Integer;

begin
  ALID:=GetLocalPupilID(DB,AID);
  if ALID=-1 then
    begin
    SID:=JLInteger.Create(AID).ToString;
    AUlog.d('Inserting pupil remote id ',SID);
    AUlog.d('Inserting pupil last name ',ALastName);
    AUlog.d('Inserting pupil last name ',ALastName);
    Q:='insert into Pupil (pu_remote_id, pu_firstname,pu_lastname) values ('+SID+','''+AFirstName+''', '''+ALastName+''')';
    DB.ExecSQL(Q);
    ALID:=GetLocalPupilID(DB,AID);
    SID:=JLInteger.Create(ALID).ToString;
    end
  else
    begin
    SID:=JLInteger.Create(ALID).ToString;
    AUlog.d('Deleting pupil groups id ',SID);
    Q:='Delete From PupilGroup where (pg_Pupil_fk='+SID+')';
    DB.ExecSQL(Q);
    end;
  AGID:=GetLocalGroupID(DB,AGroupID);
  if AGID>=0 then
    begin
    GID:=JLInteger.Create(AGID).ToString;
    Q:='insert into PupilGroup (pg_Pupil_fk, pg_Group_fk) values ('+SID+' , '+GID+')';
    DB.ExecSQL(Q);
    end;
end;

Function TAbsenteeDBHelper.GetLocalPupilID(DB : ADSSqliteDatabase; ARemoteID : Integer) : Integer;

Var
  Q : String;
  cur: ADCursor;
  SID : String;

begin
  SID:=JLLong.Create(ARemoteID).toString;
  Q:='select pu_id from pupil where (pu_remote_id='+SID+')';
  cur:=DB.rawQuery(Q,Nil);
  if (cur.getCount<>1) then
    Result:=-1
  else
    begin
    cur.moveToFirst;
    Result:=cur.getInt(0);
    end;
  Cur.close();
end;

Function TAbsenteeDBHelper.GetLocalGroupID(DB : ADSSqliteDatabase; ARemoteID : Integer) : Integer;

Var
  Q : String;
  cur: ADCursor;
  SID : String;

begin
  SID:=JLLong.Create(ARemoteID).toString;
  Q:='select gr_id from groups where (gr_remote_id='+SID+')';
  cur:=DB.rawQuery(Q,Nil);
  if cur.getCount<>1 then
    Result:=-1
  else
    begin
    cur.moveToFirst;
    Result:=cur.getInt(0);
    end;
  Cur.close();
end;

procedure TAbsenteeDBHelper.ImportGroup(DB : ADSSqliteDatabase; AID : Integer; AName : string);

Var
  Q : String;
  gid : Integer;
  SID : String;

begin
  gid:=GetLocalGroupID(DB,AID);
  if Gid=-1 then
    begin
    SID:=JLLong.Create(AID).toString;
    Q:='insert into groups (gr_remote_id, gr_name) values ('+SID+','''+AName+''')';
    DB.ExecSQL(Q);
    end;
end;

procedure TAbsenteeDBHelper.ImportPupils(AJSON: String);

Var
  j,p : OJJSONObject;
  a : OJJSONArray;
  I : Integer;
  DB : ADSSqliteDatabase;

begin
  J:=OJJSONObject.Create(AJSON);
  a:=J.getJSONArray('data');
  if A.Length>0 then
    begin
    DB:=GetWritableDatabase;
    for i:=0 to a.length-1 do
      begin
      P:=a.getJSONObject(i);
      ImportPupil(DB,P.getInt('ID'),P.GetString('FirstName'),P.getString('LastName'),P.getInt('GroupID'));
      end;
    end;
end;

procedure TAbsenteeDBHelper.ImportGroups(AJSON: String);

Var
  j,p : OJJSONObject;
  a : OJJSONArray;
  I : Integer;
  DB : ADSSqliteDatabase;

begin
  J:=OJJSONObject.Create(AJSON);
  a:=J.getJSONArray('data');
  if A.Length>0 then
    begin
    DB:=GetWritableDatabase;
    for i:=0 to a.length-1 do
      begin
      P:=a.getJSONObject(i);
      ImportGroup(DB,P.getInt('ID'),P.GetString('Name'));
      end;
    end;
end;

end.

