unit imageloader;

{$mode objfpc}{$H+}

interface

uses
  androidr14;

Type
  TFileCache = Class
  private
    FCacheDir : JIFile;
  Public
    Constructor create(AContext : ACContext);
    Function GetFile(URL : String) : JIFile;
    Procedure Clear;
  end;

  TMemoryCache = Class
  private
    FCache : JUMap;
  Public
    Constructor Create;
    Function Get (AID : String) : AGBitmap;
    Procedure Put(AID : String; ABitMap : AGBitmap);
    Procedure Clear;
   end;

implementation

Procedure CopyStream(Ins : JIInputStream; Ous: JIOutputStream);

Const
  buffer_size=1024;

Var
  Bytes : Array[1..1024] of jbyte;
  Count : Integer;

begin
  try
    While True do
      begin
      count:=ins.read(bytes,0,buffer_size);
      if (count=-1) then
         break;
      ous.write(bytes, 0, count);
      end;
  except
    // Ignore

  end;
end;

Constructor TFileCache.Create(AContext : ACContext);

begin
  //Find the dir to save cached images
  if (AOEnvironment.getExternalStorageState().equals(JLString.Create(AOEnvironment.MEDIA_MOUNTED))) then
    FCacheDir:=JIFile.Create(AOEnvironment.getExternalStorageDirectory(),'PupilImages')
  else
    FCacheDir:=AContext.getCacheDir();
  if (not FCacheDir.Exists()) then
    FCacheDir.mkdirs();
end;

Function TFileCache.getFile(url : String) : JIFile;

Var
  AFileName : String;

begin
  AFileName:=JNURLEncoder.encode(URL);
  Result:=JIFile.Create(FCacheDir,AFilename);
end;

Procedure TFileCache.Clear;

Var
  Files : Arr1JIFile;
  F : JIFile;

begin
  Files:=FCacheDir.listFiles();
  if (files=Nil) then
    exit;
  For F in Files do
    F.delete();
end;

Constructor TMemoryCache.Create;

begin
  FCache:=JUCOllections.SynchronizedMap(JUHashmap.Create);
end;

Function TMemoryCache.get(AID : String) : AGBitmap;

Var
  ref : JLRSoftReference;
  SID : JLString;

begin
  SID:=JLString.Create(Aid);
  Result:=Nil;
  if (FCache.ContainsKey(SID)) then
    begin
    Ref:=JLRSoftReference(FCache.get(SID));
    Result:=AGBitMap(ref.get());
    end;
end;

Procedure TMemoryCache.Put(AID : String; ABitmap : AGbitmap);

begin
  FCache.put(JLString.Create(AID),JLRSoftReference.Create(ABitmap));
end;

Procedure TMemoryCache.clear();
begin
  Fcache.clear();
end;

(*
public class ImageLoader {

    MemoryCache memoryCache=new MemoryCache();
    FileCache fileCache;
    private Map&lt;ImageView, String&gt; imageViews=Collections.synchronizedMap(new WeakHashMap&lt;ImageView, String&gt;());
    ExecutorService executorService;

    public ImageLoader(Context context){
        fileCache=new FileCache(context);
        executorService=Executors.newFixedThreadPool(5);
    }

    final int stub_id = R.drawable.no_image;
    public void DisplayImage(String url, ImageView imageView)
    {
        imageViews.put(imageView, url);
        Bitmap bitmap=memoryCache.get(url);
        if(bitmap!=null)
            imageView.setImageBitmap(bitmap);
        else
        {
            queuePhoto(url, imageView);
            imageView.setImageResource(stub_id);
        }
    }

    private void queuePhoto(String url, ImageView imageView)
    {
        PhotoToLoad p=new PhotoToLoad(url, imageView);
        executorService.submit(new PhotosLoader(p));
    }

    private Bitmap getBitmap(String url)
    {
        File f=fileCache.getFile(url);

        //from SD cache
        Bitmap b = decodeFile(f);
        if(b!=null)
            return b;

        //from web
        try {
            Bitmap bitmap=null;
            URL imageUrl = new URL(url);
            HttpURLConnection conn = (HttpURLConnection)imageUrl.openConnection();
            conn.setConnectTimeout(30000);
            conn.setReadTimeout(30000);
            conn.setInstanceFollowRedirects(true);
            InputStream is=conn.getInputStream();
            OutputStream os = new FileOutputStream(f);
            Utils.CopyStream(is, os);
            os.close();
            bitmap = decodeFile(f);
            return bitmap;
        } catch (Exception ex){
           ex.printStackTrace();
           return null;
        }
    }

    //decodes image and scales it to reduce memory consumption
    private Bitmap decodeFile(File f){
        try {
            //decode image size
            BitmapFactory.Options o = new BitmapFactory.Options();
            o.inJustDecodeBounds = true;
            BitmapFactory.decodeStream(new FileInputStream(f),null,o);

            //Find the correct scale value. It should be the power of 2.
            final int REQUIRED_SIZE=70;
            int width_tmp=o.outWidth, height_tmp=o.outHeight;
            int scale=1;
            while(true){
                if(width_tmp/2&lt;REQUIRED_SIZE || height_tmp/2&lt;REQUIRED_SIZE)
                    break;
                width_tmp/=2;
                height_tmp/=2;
                scale*=2;
            }

            //decode with inSampleSize
            BitmapFactory.Options o2 = new BitmapFactory.Options();
            o2.inSampleSize=scale;
            return BitmapFactory.decodeStream(new FileInputStream(f), null, o2);
        } catch (FileNotFoundException e) {}
        return null;
    }

    //Task for the queue
    private class PhotoToLoad
    {
        public String url;
        public ImageView imageView;
        public PhotoToLoad(String u, ImageView i){
            url=u;
            imageView=i;
        }
    }

    class PhotosLoader implements Runnable {
        PhotoToLoad photoToLoad;
        PhotosLoader(PhotoToLoad photoToLoad){
            this.photoToLoad=photoToLoad;
        }

        @Override
        public void run() {
            if(imageViewReused(photoToLoad))
                return;
            Bitmap bmp=getBitmap(photoToLoad.url);
            memoryCache.put(photoToLoad.url, bmp);
            if(imageViewReused(photoToLoad))
                return;
            BitmapDisplayer bd=new BitmapDisplayer(bmp, photoToLoad);
            Activity a=(Activity)photoToLoad.imageView.getContext();
            a.runOnUiThread(bd);
        }
    }

    boolean imageViewReused(PhotoToLoad photoToLoad){
        String tag=imageViews.get(photoToLoad.imageView);
        if(tag==null || !tag.equals(photoToLoad.url))
            return true;
        return false;
    }

    //Used to display bitmap in the UI thread
    class BitmapDisplayer implements Runnable
    {
        Bitmap bitmap;
        PhotoToLoad photoToLoad;
        public BitmapDisplayer(Bitmap b, PhotoToLoad p){bitmap=b;photoToLoad=p;}
        public void run()
        {
            if(imageViewReused(photoToLoad))
                return;
            if(bitmap!=null)
                photoToLoad.imageView.setImageBitmap(bitmap);
            else
                photoToLoad.imageView.setImageResource(stub_id);
        }
    }

    public void clearCache() {
        memoryCache.clear();
        fileCache.clear();
    }

}
*)
end.

